/************************************************************************
 *
 * 24Cxx version 1.0 - Copyright (C) 2000 Luca Ferretti <luca.ferretti@wind.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *            See the file COPYING for more details.
 *
 *************************************************************************
 *
 * PROGRAM : 24Cxxx.h
 * RELEASE : 1.0
 * DATE    : 17 Sep 2000
 *
 ************************************************************************/

// System include
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <time.h>
#include <search.h>
#include <sched.h>

// K defs
#define VERSION		"1.0"
#define K_BIT		16		// Default
#define B_SIZE		( K_BIT * 1024 ) / 8 	// R U a Genius ?
#define DEFAULT_FNAME	"./eeprom.bin"
#define DEFAULT_PORT	"/dev/ttyS0"
#define MAX_E2PROM_SIZE 65536		// 512K is,AFAIK,the biggest one
#define POWERUP_DELAY	500000		// uS between power on & ops
#define TXX		0	 	// Bit ops timing (dummy loop)
#define WCT		10000		// uS between each pgm cycle
					// This is the SAFEST SETTING possible
					// w/o waste of time.

#define SLAVE_ADDR	0xa0		// identify a serial eeprom
					// (the hi nibble)
#define BLOCK_SEL	0x00		// 3 bits
#define READ_BIT    	0x01		// lsb that identify
#define WRITE_BIT   	0x00		//   op type
#define R_HEADER	SLAVE_ADDR + BLOCK_SEL + READ_BIT
#define W_HEADER	SLAVE_ADDR + BLOCK_SEL + WRITE_BIT

#define E2PROMS	 { "1","2","4","8","16","32","64","128","256","512" }
#define PG_SIZES {  8,  8,  16, 16, 16,  32,  32,  32,   64,   32 }

#define vcc_on	(ioctl (fd, TIOCSBRK, 0)) 	// TxD --> Vcc
#define vcc_off	(ioctl (fd, TIOCCBRK, 0))
#define scl_lo	( ioctl (fd,TIOCMBIC,&rts) )	// RTS --> SCL
#define scl_hi	( ioctl (fd,TIOCMBIS,&rts) )
#define sda_lo	( ioctl (fd,TIOCMBIC,&dtr) )	// DTR --> SDA
#define sda_hi	( ioctl (fd,TIOCMBIS,&dtr) )

#define	W_BIT_MASK	0x01			// Options
#define	R_BIT_MASK	0x02			//  bits
#define	V_BIT_MASK	0x04			//   masks

// Prototypes defs
int open_port(char * port);
void idle(void);
void delay(int delay);
unsigned char recv_bit(void);			// CTS <-- SDA
unsigned char recv_ack(void);
void send_ack(void);
void send_start(void);
void send_stop(void);
void send_byte(unsigned char byte);
unsigned char recv_byte(void);
void read_e2prom(unsigned char *buffer,int eeprom_size,int size);
void write_e2prom(unsigned char *buffer,int eeprom_size,int size);
int get_pg_size(char *KB);
void save_buffer(char *f_name,unsigned char *buffer,int size);
int read_buffer(char *f_name,unsigned char *buffer);
void usage(void);
void progress(void);
void schedulize_srt(void);
// Glob vars defs
const char rts=TIOCM_RTS;
const char dtr=TIOCM_DTR;
int wct=WCT;					// write cycle time
int pg_size=8;					// wr pg size
int fd;						// serial line fd
unsigned char options=0;			// cmd line options status reg
int progress_val=0;				// whirley counter
char header[]="\n---------------------------------------------------------------------------\n   24Cxxx Version " VERSION " (C)FLS Luca Ferretti Systems <luca.ferretti@wind.it>\n---------------------------------------------------------------------------\n   24Cxxx comes with ABSOLUTELY NO WARRANTY,\n   This is free software, and you are welcome to redistribute it\n   under certain conditions; see the file COPYING for details.\n---------------------------------------------------------------------------\n\n";
// end of 24Cxxx.h
